package install

import (
	"fmt"
	"go/build"
	"os"
	"os/exec"
	"path/filepath"

	"github.com/gobuffalo/cli/internal/genny/add"
	"github.com/gobuffalo/cli/internal/plugins/plugdeps"
	"github.com/gobuffalo/genny/v2"
)

// New installs plugins and then added them to the config file
func New(opts *Options) (*genny.Group, error) {
	gg := &genny.Group{}

	if err := opts.Validate(); err != nil {
		return gg, err
	}

	aopts := &add.Options{
		App:     opts.App,
		Plugins: opts.Plugins,
	}

	if err := aopts.Validate(); err != nil {
		return gg, err
	}

	g := genny.New()
	proot := filepath.Join(opts.App.Root, "plugins")
	for _, p := range opts.Plugins {
		if len(p.GoGet) == 0 {
			continue
		}

		var args []string
		if len(p.Tags) > 0 {
			args = append(args, "-tags", p.Tags.String())
		}
		bargs := []string{"install"}
		bargs = append(bargs, args...)
		bargs = append(bargs, p.GoGet)
		g.Command(exec.Command("go", bargs...))
		if opts.Vendor {
			g.RunFn(pRun(proot, p))
		}
	}
	gg.Add(g)

	g, err := add.New(aopts)
	if err != nil {
		return gg, err
	}

	gg.Add(g)

	return gg, nil
}

func pRun(proot string, p plugdeps.Plugin) genny.RunFn {
	return func(r *genny.Runner) error {
		c := build.Default
		if c.GOOS == "windows" {
			return fmt.Errorf("vendoring of plugins is currently not supported on windows. PRs are VERY welcome! :)")
		}

		bp := filepath.Join(c.GOPATH, "bin", p.Binary)
		sf, err := r.FindFile(bp)
		if err != nil {
			return err
		}

		pbp := filepath.Join(proot, p.Binary)
		if err := r.Disk.Delete(pbp); err != nil {
			return err
		}

		df := genny.NewFile(pbp, sf)
		if err := r.File(df); err != nil {
			return err
		}

		return os.Chmod(pbp, 0o555)
	}
}
